#!/usr/bin/env python
#-*- coding: utf-8 -*-
"""
This module provides following basic data processing functions:
    LinBg(Y, X)         - determine the linear background of data
    RmBg(X, Y)          - determine and subtract the linear background 
    Integr(X, Y)        - trapezoid integration
	FirstMoment(X, Y)   - i. e. center of mass of peak (note: you may want to call RmBg() first)
    SecondMoment(X, Y)  - i. e. std. deviation of peak
    PeakFinder(X, Y)    - returns list of (X,Y) tuples with points, that are highest in their neighborhood

The functions work both with lists of floats and with numpy.arrays.

Copyright (c) 2011-02-14 Filip Dominec, dominecf at fzu cz
This software is free as speech after five beers.  Published under BSD license. 
See attached file or http://opensource.org/licenses/bsd-license.php for details.
"""

import numpy 
def LinBg(X, Y, AvgPoints=2):#{{{
	""" 
	Determines the linear background (e. g. of some peak). Uses some number (=AvgPoints) of points from the side.
	Returns a tuple containing:
	1) the linear background values for Y, 
	2) its slope and 
	3) its offset at x=0.
	>>> LinBg(range(5), [10,11,100,13,14], AvgPoints=2)
	(array([ 10.,  11.,  12.,  13.,  14.]), 1.0, 10.0)
	"""
	assert len(Y) >= AvgPoints
	AvgY1 = sum(Y[0:AvgPoints])/float(AvgPoints) # todo: use weights by X spacing
	AvgY2 = sum(Y[-AvgPoints:])/float(AvgPoints) 
	AvgX1 = sum(X[0:AvgPoints])/float(AvgPoints) 
	AvgX2 = sum(X[-AvgPoints:])/float(AvgPoints) 
	slope = (AvgY2-AvgY1)/(AvgX2-AvgX1)
	offset = AvgY1 - AvgX1*slope 
	return (numpy.add(numpy.multiply(X, slope), offset), slope, offset)
#}}}
def RmBg(X, Y, **kwargs):#{{{
	""" 
	Removes the linear background (e. g. of some peak). Utilizes the LinBg() function. 

	>>> RmBg(range(5), [10,11,100,13,14], AvgPoints=2)
	[0, 0, 88, 0, 0], 1, 10
	"""
	GotLists = (type(Y) == list)
	Y = numpy.array(Y) 

	YBg, slope, offset = LinBg(X, Y, kwargs)
	output=numpy.subtract(Y, YBg)
	if GotLists: output = list(output)
	return output, slope, offset
	#}}}
def Integr(X, Y):#{{{
	"""
	Trapezoid integration.
	>>> Integr([1,5,7], [10,20,0])
	80.0
	"""
	X = numpy.array(X); Y = numpy.array(Y)
	assert len(X) == len(Y)
	return numpy.dot(X[1:]-X[0:-1], Y[1:] + Y[0:-1]) / 2.0
	# GotLists = (type(X) == list and type(Y) == list)
	# if GotLists: output = list(output)
	# return output
#}}}
def FirstMoment(X, Y):#{{{
	"""
	Returns the first moment (center of mass) on the X axis.
	>>> FirstMoment([1,3,5], [0,8,0])
	3.0
	"""
	assert len(X) == len(Y)
	if len(X) == 1: return X[0]
	Y = numpy.array(Y); X = numpy.array(X)
	return Integr(Y*X, X)/Integr(Y,X)
#}}}
def SecondMoment(X, Y):#{{{
	"""
	Returns the second moment (~ peak half-width squared) on the X axis. May be written also as ⟨X²⟩-⟨X⟩².
	"""
	area = Integr(X, Y)
	return Integr(X, Y*(X**2))/area - (Integr(X, Y*X)/area)**2
#}}}
def PeakFinder(X, Y, surrpoints=2):#{{{
	"""
	Finds the points, that are higher than all points it their neighborhood (the maximum distance is given by 'surrpoints').
	Omits the points at the boundary.
	Todo: surrpoints shall be given not by number of points, but by X distance.
	>>> PeakFinder(range(10), [0,0,0,1,2,1,3,2,1,0], surrpoints=2)
	[[3, 6]]
	>>> PeakFinder(range(10), [0,0,0,1,2,1,3,2,1,0], surrpoints=1)
	[[2, 4], [3, 6]]
	"""
	GotLists = (type(X) == list and type(Y) == list)
	PeakList = []
	for (myN, myX, myY) in  zip(range(surrpoints,len(X)-surrpoints), X[surrpoints:-surrpoints], Y[surrpoints:-surrpoints]):
		if not filter(lambda Y: Y>=myY, Y[myN-surrpoints:myN]) and not filter(lambda Y: Y>=myY, Y[myN+1:myN+surrpoints+1]):
			PeakList.append([myY,myX]) # we are the best in this range!
	return PeakList
#}}}

if __name__ == '__main__':		# avoid execution if loaded as a module only
	try:
		print('Testing the module now.')
		import doctest

		doctest.testmod()
	except:
		traceback.print_exc()
		print >>sys.stderr
		print >>sys.stderr, "Press Enter to exit."
		blah = raw_input()	# enables to read the error report on volatile terminal


