#!/usr/bin/python

import argparse
import numpy as np
import dataproc
from scipy import constants
from scipy.optimize import leastsq
from scipy.signal import lombscargle
import matplotlib.pyplot as plt

def vel(freq, freq0):
    """return velocity scale"""
    return constants.c*1e-3 * (freq0-freq)/freq0

def fitfunc(p, x, freq): 
    """Target function"""
    sinwave = 0.
    for i in range(len(p)/2):
        sinwave += p[2*i]*np.sin(freq[i]*x) + p[2*i+1]*np.cos(freq[i]*x)
    return sinwave

# Parsing command line arguments
parser = argparse.ArgumentParser()
parser.add_argument('-f', '--file', help='ASCII input file')
parser.add_argument('--freq', default='1097.3647910', type=float, 
        help='Line frequency [GHz]')
parser.add_argument('--width', default=16, type=float,
        help='line width [km/s]')
parser.add_argument('-n', default=20, type=int, help='Number of sine waves')
args = parser.parse_args()

# Read frequency and flux arrays
Freq, Flux = np.loadtxt(args.file, unpack=True)
# exclude nans
Freq = Freq[~np.isnan(Flux)]
Flux = Flux[~np.isnan(Flux)]
# baseline mask excluding the line
Vel = vel(Freq, args.freq)
mask = np.where(np.abs(Vel) > args.width)
x = Freq[mask]
y = Flux[mask] - np.mean(Flux[mask])

# frequency array
f = np.linspace(0.05, 1e3, 1e4)
# For normalization of the periodogram
pgram = lombscargle(x, y, f)

# Find stronger peaks
PeakList = dataproc.PeakFinder(f, pgram, surrpoints=2)
PeakInd = np.argsort(np.array(PeakList)[:,0])[-args.n:]
freq = np.array(PeakList)[np.array(PeakInd)][:,1]
# Fit using least squares
errfunc = lambda p, x, y: fitfunc(p, x, freq) - y # Distance to the target function
p0 = np.ones(2*args.n) # Initial guess for the parameters
p1, success = leastsq(errfunc, p0, args=(x, y))

# write results
np.savetxt('lomb_output.txt', 
        np.transpose((Freq, Flux - fitfunc(p1, Freq, freq))))

plt.subplot(2, 1, 1)
# plot masked baseline
plt.plot(Vel, Flux, drawstyle='steps-mid')
plt.plot(Vel, fitfunc(p1, Freq, freq) + np.mean(Flux[mask]), 'red')
plt.subplot(2, 1, 2)
plt.plot(Vel, Flux - fitfunc(p1, Freq, freq))
plt.show()
